#!/usr/bin/python
import urllib
import json
import sys

LATEST_VERSION_URI = 'https://api.bintray.com/packages/:user/:repository/:package/versions/_latest'
VERSION_FILE_LIST_URI = 'https://api.bintray.com/packages/:user/:repository/:package/versions/:version/files'
DOWNLOAD_URI = 'https://dl.bintray.com/:user/:repository/:file_path'

def getURL(uri):
    response = urllib.urlopen(uri)
    return response.read()

def downloadURLInto(uri, destination):
    def reportHook(count, blockSize, totalSize):
        downloaded = count*blockSize
        percent = int(downloaded*100/totalSize)
        sys.stdout.write('Download %s %3d%%[%d / %d KB]\r' % (destination, percent, downloaded/1024, totalSize/1024))
        sys.stdout.flush()

    urllib.urlretrieve(uri, destination, reportHook)
    sys.stdout.write('\n')
    sys.stdout.flush()

def requestJSON(uri):
    return json.loads(getURL(uri))

def getPackageLatestVersionInfo(user, repository, package):
    uri = LATEST_VERSION_URI \
        .replace(':user', user) \
        .replace(':repository', repository) \
        .replace(':package', package)
    return requestJSON(uri)

def getVersionFileList(user, repository, package, version):
    uri = VERSION_FILE_LIST_URI \
        .replace(':user', user) \
        .replace(':repository', repository) \
        .replace(':package', package) \
        .replace(':version', version)
    return requestJSON(uri)

def getLatestVersionFileList(user, repository, package):
    versionInfo = getPackageLatestVersionInfo(user, repository, package)
    return list(map(lambda e: e['path'], getVersionFileList(user, repository, package, versionInfo['name'])))

def getLatestVersionForVariant(user, repository, package, variant):
    fileList = getLatestVersionFileList(user, repository, package)
    return list(filter(lambda e: variant in e, fileList))

def downloadLatestVersionForVariant(user, repository, package, variant):
    fileName = getLatestVersionForVariant(user, repository, package, variant)[0]
    downloadUri = DOWNLOAD_URI \
        .replace(':user', user) \
        .replace(':repository', repository) \
        .replace(':file_path', fileName)
    downloadURLInto(downloadUri, fileName)
    return fileName

COG_USER = 'opensmalltalk'
COG_REPOSITORY = 'vm'
COG_PACKAGE = 'cog'

def bintray_cogPrintHelp():
    print """bintray_cog.py [options] <vm-variant>
-filename       Prints the file names for the VM variant. [Default]
-url            Prints the download url for the VM variant.
-download       Downloads the specified VM variant. This prints the downloaded file name on success.
-help           Prints this message
"""

def bintray_cogPrintFileNames(variant):
    results = getLatestVersionForVariant(COG_USER, COG_REPOSITORY, COG_PACKAGE, variant)
    for result in results:
        print result


def bintray_cogPrintURLAction(variant):
    results = getLatestVersionForVariant(COG_USER, COG_REPOSITORY, COG_PACKAGE, variant)
    for result in results:
        downloadUri = DOWNLOAD_URI \
            .replace(':user', COG_USER) \
            .replace(':repository', COG_REPOSITORY) \
            .replace(':file_path', result)
        print downloadUri

def bintray_cogDownloadAction(variant):
    results = getLatestVersionForVariant(COG_USER, COG_REPOSITORY, COG_PACKAGE, variant)
    if len(results) == 0:
        sys.exit(1)

    filename = results[0]
    downloadUri = DOWNLOAD_URI \
        .replace(':user', COG_USER) \
        .replace(':repository', COG_REPOSITORY) \
        .replace(':file_path', filename)
    downloadURLInto(downloadUri, filename)
    print filename

def bintray_cogMain():
    i = 1;
    variant = None
    action = bintray_cogPrintFileNames
    while i < len(sys.argv):
        arg = sys.argv[i]
        if arg.startswith('-'):
            if arg == '-filename':
                action = bintray_cogPrintFileNames
            if arg == '-url':
                action = bintray_cogPrintURLAction
            elif arg == '-download':
                action = bintray_cogDownloadAction
            elif arg == '-help':
                bintray_cogPrintHelp()
                sys.exit(0)
            else:
                sys.exit(1)
        else:
            variant = arg
        i += 1

    if variant is None:
        bintray_cogPrintHelp()
        sys.exit(1)
    action(variant)

if __name__ == '__main__':
    bintray_cogMain()

